/*
 * The Relaxer artifact
 * Copyright (c) 2000-2004, ASAMI Tomoharu, All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer. 
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package galatea.relaxer.event;

import java.io.*;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Writer;
import java.net.URL;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.*;

/**
 * <b>EventInterpreted</b> is generated from event.rng by Relaxer.
 * This class is derived from:
 * 
 * <!-- for programmer
 * <element name="interpreted">
 *      <optional>
 *       <element name="text">
 *         <data type="token"/>
 *       </element>
 *      </optional>
 *      <optional>
 *       <element name="script">
 *         <data type="token"/>
 *       </element>
 *      </optional>
 *     </element>-->
 * <!-- for javadoc -->
 * <pre> &lt;element name="interpreted"&gt;
 *      &lt;optional&gt;
 *       &lt;element name="text"&gt;
 *         &lt;data type="token"/&gt;
 *       &lt;/element&gt;
 *      &lt;/optional&gt;
 *      &lt;optional&gt;
 *       &lt;element name="script"&gt;
 *         &lt;data type="token"/&gt;
 *       &lt;/element&gt;
 *      &lt;/optional&gt;
 *     &lt;/element&gt;</pre>
 *
 * @version event.rng (Tue Jul 06 15:45:52 JST 2004)
 * @author  Relaxer 1.1b (http://www.relaxer.org)
 */
public class EventInterpreted implements java.io.Serializable, Cloneable, IEventEvChoice {
    private String text_;
    private String script_;

    /**
     * Creates a <code>EventInterpreted</code>.
     *
     */
    public EventInterpreted() {
    }

    /**
     * Creates a <code>EventInterpreted</code>.
     *
     * @param source
     */
    public EventInterpreted(EventInterpreted source) {
        setup(source);
    }

    /**
     * Creates a <code>EventInterpreted</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public EventInterpreted(RStack stack) {
        setup(stack);
    }

    /**
     * Creates a <code>EventInterpreted</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public EventInterpreted(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Creates a <code>EventInterpreted</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public EventInterpreted(Element element) {
        setup(element);
    }

    /**
     * Creates a <code>EventInterpreted</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public EventInterpreted(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file);
    }

    /**
     * Creates a <code>EventInterpreted</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public EventInterpreted(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(uri);
    }

    /**
     * Creates a <code>EventInterpreted</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public EventInterpreted(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(url);
    }

    /**
     * Creates a <code>EventInterpreted</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public EventInterpreted(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(in);
    }

    /**
     * Creates a <code>EventInterpreted</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public EventInterpreted(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(is);
    }

    /**
     * Creates a <code>EventInterpreted</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public EventInterpreted(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(reader);
    }

    /**
     * Initializes the <code>EventInterpreted</code> by the EventInterpreted <code>source</code>.
     *
     * @param source
     */
    public void setup(EventInterpreted source) {
        int size;
        text_ = source.text_;
        script_ = source.script_;
    }

    /**
     * Initializes the <code>EventInterpreted</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public void setup(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Initializes the <code>EventInterpreted</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public void setup(Element element) {
        init(element);
    }

    /**
     * Initializes the <code>EventInterpreted</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public void setup(RStack stack) {
        init(stack.popElement());
    }

    /**
     * @param element
     */
    private void init(Element element) {
        RStack stack = new RStack(element);
        text_ = URelaxer.getElementPropertyAsStringByStack(stack, "text");
        script_ = URelaxer.getElementPropertyAsStringByStack(stack, "script");
    }

    /**
     * @return Object
     */
    public Object clone() {
        return (new EventInterpreted((EventInterpreted)this));
    }

    /**
     * Creates a DOM representation of the object.
     * Result is appended to the Node <code>parent</code>.
     *
     * @param parent
     */
    public void makeElement(Node parent) {
        Document doc;
        if (parent instanceof Document) {
            doc = (Document)parent;
        } else {
            doc = parent.getOwnerDocument();
        }
        Element element = doc.createElement("interpreted");
        int size;
        if (this.text_ != null) {
            URelaxer.setElementPropertyByString(element, "text", this.text_);
        }
        if (this.script_ != null) {
            URelaxer.setElementPropertyByString(element, "script", this.script_);
        }
        parent.appendChild(element);
    }

    /**
     * Initializes the <code>EventInterpreted</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file.toURL());
    }

    /**
     * Initializes the <code>EventInterpreted</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(uri, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>EventInterpreted</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(url, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>EventInterpreted</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(in, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>EventInterpreted</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(is, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>EventInterpreted</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(reader, UJAXP.FLAG_NONE));
    }

    /**
     * Creates a DOM document representation of the object.
     *
     * @exception ParserConfigurationException
     * @return Document
     */
    public Document makeDocument() throws ParserConfigurationException {
        Document doc = UJAXP.makeDocument();
        makeElement(doc);
        return (doc);
    }

    /**
     * Gets the String property <b>text</b>.
     *
     * @return String
     */
    public String getText() {
        return (text_);
    }

    /**
     * Sets the String property <b>text</b>.
     *
     * @param text
     */
    public void setText(String text) {
        this.text_ = text;
    }

    /**
     * Gets the String property <b>script</b>.
     *
     * @return String
     */
    public String getScript() {
        return (script_);
    }

    /**
     * Sets the String property <b>script</b>.
     *
     * @param script
     */
    public void setScript(String script) {
        this.script_ = script;
    }

    /**
     * Makes an XML text representation.
     *
     * @return String
     */
    public String makeTextDocument() {
        StringBuffer buffer = new StringBuffer();
        makeTextElement(buffer);
        return (new String(buffer));
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(StringBuffer buffer) {
        int size;
        buffer.append("<interpreted");
        buffer.append(">");
        if (text_ != null) {
            buffer.append("<text>");
            buffer.append(URelaxer.escapeCharData(URelaxer.getString(getText())));
            buffer.append("</text>");
        }
        if (script_ != null) {
            buffer.append("<script>");
            buffer.append(URelaxer.escapeCharData(URelaxer.getString(getScript())));
            buffer.append("</script>");
        }
        buffer.append("</interpreted>");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextElement(Writer buffer) throws IOException {
        int size;
        buffer.write("<interpreted");
        buffer.write(">");
        if (text_ != null) {
            buffer.write("<text>");
            buffer.write(URelaxer.escapeCharData(URelaxer.getString(getText())));
            buffer.write("</text>");
        }
        if (script_ != null) {
            buffer.write("<script>");
            buffer.write(URelaxer.escapeCharData(URelaxer.getString(getScript())));
            buffer.write("</script>");
        }
        buffer.write("</interpreted>");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(PrintWriter buffer) {
        int size;
        buffer.print("<interpreted");
        buffer.print(">");
        if (text_ != null) {
            buffer.print("<text>");
            buffer.print(URelaxer.escapeCharData(URelaxer.getString(getText())));
            buffer.print("</text>");
        }
        if (script_ != null) {
            buffer.print("<script>");
            buffer.print(URelaxer.escapeCharData(URelaxer.getString(getScript())));
            buffer.print("</script>");
        }
        buffer.print("</interpreted>");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(StringBuffer buffer) {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextAttribute(Writer buffer) throws IOException {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(PrintWriter buffer) {
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getTextAsString() {
        return (URelaxer.getString(getText()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getScriptAsString() {
        return (URelaxer.getString(getScript()));
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setTextByString(String string) {
        setText(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setScriptByString(String string) {
        setScript(string);
    }

    /**
     * Returns a String representation of this object.
     * While this method informs as XML format representaion, 
     *  it's purpose is just information, not making 
     * a rigid XML documentation.
     *
     * @return String
     */
    public String toString() {
        try {
            return (makeTextDocument());
        } catch (Exception e) {
            return (super.toString());
        }
    }

    /**
     * Tests if a Element <code>element</code> is valid
     * for the <code>EventInterpreted</code>.
     *
     * @param element
     * @return boolean
     */
    public static boolean isMatch(Element element) {
        if (!URelaxer.isTargetElement(element, "interpreted")) {
            return (false);
        }
        RStack target = new RStack(element);
        boolean $match$ = false;
        Element child;
        child = target.peekElement();
        if (child != null) {
            if (URelaxer.isTargetElement(child, "text")) {
                target.popElement();
            }
        }
        $match$ = true;
        child = target.peekElement();
        if (child != null) {
            if (URelaxer.isTargetElement(child, "script")) {
                target.popElement();
            }
        }
        $match$ = true;
        if (!target.isEmptyElement()) {
            return (false);
        }
        return (true);
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>EventInterpreted</code>.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatch(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        return (isMatch(element));
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>EventInterpreted</code>.
     * This method consumes the stack contents during matching operation.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatchHungry(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        if (isMatch(element)) {
            stack.popElement();
            return (true);
        } else {
            return (false);
        }
    }
}
