/*
 * The Relaxer artifact
 * Copyright (c) 2000-2004, ASAMI Tomoharu, All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer. 
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package galatea.relaxer.vxml20;

import java.io.*;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Writer;
import java.net.URL;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.*;

/**
 * <b>VxmlFilled</b> is generated from vxml20.rng by Relaxer.
 * This class is derived from:
 * 
 * <!-- for programmer
 * <element name="filled">
 *       <optional>
 *         <attribute name="mode">
 *           <choice>
 *             <value type="NMTOKEN">any</value>
 *             <value type="NMTOKEN">all</value>
 *           </choice>
 *         </attribute>
 *       </optional>
 *       <optional>
 *         <attribute name="namelist">
 *           <text/>
 *         </attribute>
 *       </optional>
 *       <mixed>
 *         <zeroOrMore>
 *           <choice>
 *             <ref name="submit"/>
 *             <ref name="goto"/>
 *             <ref name="audio"/>
 *             <ref name="prompt"/>
 *             <ref name="script"/>
 *             <ref name="throw"/>
 *             <ref name="log"/>
 *             <ref name="native"/>
 *             <ref name="assign"/>
 *             <ref name="exit"/>
 *             <ref name="return"/>
 *             <ref name="var"/>
 *             <ref name="value"/>
 *             <ref name="clear"/>
 *             <ref name="if"/>
 *             <ref name="enumerate"/>
 *             <ref name="disconnect"/>
 *             <ref name="reprompt"/>
 *           </choice>
 *         </zeroOrMore>
 *       </mixed>
 *     </element>-->
 * <!-- for javadoc -->
 * <pre> &lt;element name="filled"&gt;
 *       &lt;optional&gt;
 *         &lt;attribute name="mode"&gt;
 *           &lt;choice&gt;
 *             &lt;value type="NMTOKEN"&gt;any&lt;/value&gt;
 *             &lt;value type="NMTOKEN"&gt;all&lt;/value&gt;
 *           &lt;/choice&gt;
 *         &lt;/attribute&gt;
 *       &lt;/optional&gt;
 *       &lt;optional&gt;
 *         &lt;attribute name="namelist"&gt;
 *           &lt;text/&gt;
 *         &lt;/attribute&gt;
 *       &lt;/optional&gt;
 *       &lt;mixed&gt;
 *         &lt;zeroOrMore&gt;
 *           &lt;choice&gt;
 *             &lt;ref name="submit"/&gt;
 *             &lt;ref name="goto"/&gt;
 *             &lt;ref name="audio"/&gt;
 *             &lt;ref name="prompt"/&gt;
 *             &lt;ref name="script"/&gt;
 *             &lt;ref name="throw"/&gt;
 *             &lt;ref name="log"/&gt;
 *             &lt;ref name="native"/&gt;
 *             &lt;ref name="assign"/&gt;
 *             &lt;ref name="exit"/&gt;
 *             &lt;ref name="return"/&gt;
 *             &lt;ref name="var"/&gt;
 *             &lt;ref name="value"/&gt;
 *             &lt;ref name="clear"/&gt;
 *             &lt;ref name="if"/&gt;
 *             &lt;ref name="enumerate"/&gt;
 *             &lt;ref name="disconnect"/&gt;
 *             &lt;ref name="reprompt"/&gt;
 *           &lt;/choice&gt;
 *         &lt;/zeroOrMore&gt;
 *       &lt;/mixed&gt;
 *     &lt;/element&gt;</pre>
 *
 * @version vxml20.rng (Wed Jul 07 07:19:11 JST 2004)
 * @author  Relaxer 1.1b (http://www.relaxer.org)
 */
public class VxmlFilled implements java.io.Serializable, Cloneable, IRNSContainer, IRVisitable, IRNode, IVxmlObjectMixed, IVxmlObjectMixedChoice, IVxmlSubdialogMixed, IVxmlSubdialogMixedChoice, IVxmlRecordMixed, IVxmlRecordMixedChoice, IVxmlTransferMixed, IVxmlTransferMixedChoice, IVxmlFieldMixed, IVxmlFieldMixedChoice, IVxmlFormChoice {
    public static final String MODE_ANY = "any";
    public static final String MODE_ALL = "all";

    private RNSContext rNSContext_ = new RNSContext(this, "http://hil.t.u-tokyo.ac.jp/XML/2002/VXML");
    private String mode_;
    private String namelist_;
    // List<IVxmlFilledMixed>
    private java.util.List content_ = new java.util.ArrayList();
    private IRNode parentRNode_;

    /**
     * Creates a <code>VxmlFilled</code>.
     *
     */
    public VxmlFilled() {
    }

    /**
     * Creates a <code>VxmlFilled</code>.
     *
     * @param source
     */
    public VxmlFilled(VxmlFilled source) {
        setup(source);
    }

    /**
     * Creates a <code>VxmlFilled</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public VxmlFilled(RStack stack) {
        setup(stack);
    }

    /**
     * Creates a <code>VxmlFilled</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public VxmlFilled(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Creates a <code>VxmlFilled</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public VxmlFilled(Element element) {
        setup(element);
    }

    /**
     * Creates a <code>VxmlFilled</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public VxmlFilled(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file);
    }

    /**
     * Creates a <code>VxmlFilled</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public VxmlFilled(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(uri);
    }

    /**
     * Creates a <code>VxmlFilled</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public VxmlFilled(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(url);
    }

    /**
     * Creates a <code>VxmlFilled</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public VxmlFilled(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(in);
    }

    /**
     * Creates a <code>VxmlFilled</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public VxmlFilled(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(is);
    }

    /**
     * Creates a <code>VxmlFilled</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public VxmlFilled(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(reader);
    }

    /**
     * Initializes the <code>VxmlFilled</code> by the VxmlFilled <code>source</code>.
     *
     * @param source
     */
    public void setup(VxmlFilled source) {
        int size;
        mode_ = source.mode_;
        namelist_ = source.namelist_;
        this.content_.clear();
        size = source.content_.size();
        for (int i = 0;i < size;i++) {
            addContent((IVxmlFilledMixed)source.getContent(i).clone());
        }
    }

    /**
     * Initializes the <code>VxmlFilled</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public void setup(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Initializes the <code>VxmlFilled</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public void setup(Element element) {
        init(element);
    }

    /**
     * Initializes the <code>VxmlFilled</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public void setup(RStack stack) {
        init(stack.popElement());
    }

    /**
     * @param element
     */
    private void init(Element element) {
        RStack stack = new RStack(element);
        rNSContext_.declareNamespace(element);
        mode_ = URelaxer.getAttributePropertyAsString(element, "mode");
        namelist_ = URelaxer.getAttributePropertyAsString(element, "namelist");
        this.content_.clear();
        while (true) {
            if (RString.isMatch(stack)) {
                addContent(new RString(stack));
            } else if (VxmlSubmit.isMatch(stack)) {
                addContent(new VxmlSubmit(stack));
            } else if (VxmlGoto.isMatch(stack)) {
                addContent(new VxmlGoto(stack));
            } else if (VxmlAudio.isMatch(stack)) {
                addContent(new VxmlAudio(stack));
            } else if (VxmlPrompt.isMatch(stack)) {
                addContent(new VxmlPrompt(stack));
            } else if (VxmlScript.isMatch(stack)) {
                addContent(new VxmlScript(stack));
            } else if (VxmlThrow.isMatch(stack)) {
                addContent(new VxmlThrow(stack));
            } else if (VxmlLog.isMatch(stack)) {
                addContent(new VxmlLog(stack));
            } else if (VxmlNative.isMatch(stack)) {
                addContent(new VxmlNative(stack));
            } else if (VxmlAssign.isMatch(stack)) {
                addContent(new VxmlAssign(stack));
            } else if (VxmlExit.isMatch(stack)) {
                addContent(new VxmlExit(stack));
            } else if (VxmlReturn.isMatch(stack)) {
                addContent(new VxmlReturn(stack));
            } else if (VxmlVar.isMatch(stack)) {
                addContent(new VxmlVar(stack));
            } else if (VxmlValue.isMatch(stack)) {
                addContent(new VxmlValue(stack));
            } else if (VxmlClear.isMatch(stack)) {
                addContent(new VxmlClear(stack));
            } else if (VxmlIf.isMatch(stack)) {
                addContent(new VxmlIf(stack));
            } else if (VxmlEnumerate.isMatch(stack)) {
                addContent(new VxmlEnumerate(stack));
            } else if (VxmlDisconnect.isMatch(stack)) {
                addContent(new VxmlDisconnect(stack));
            } else if (VxmlReprompt.isMatch(stack)) {
                addContent(new VxmlReprompt(stack));
            } else {
                break;
            }
        }
    }

    /**
     * @return Object
     */
    public Object clone() {
        return (new VxmlFilled((VxmlFilled)this));
    }

    /**
     * Creates a DOM representation of the object.
     * Result is appended to the Node <code>parent</code>.
     *
     * @param parent
     */
    public void makeElement(Node parent) {
        Document doc;
        if (parent instanceof Document) {
            doc = (Document)parent;
        } else {
            doc = parent.getOwnerDocument();
        }
        Element element = doc.createElementNS("http://hil.t.u-tokyo.ac.jp/XML/2002/VXML", "filled");
        rNSContext_.setupNamespace(element);
        int size;
        if (this.mode_ != null) {
            URelaxer.setAttributePropertyByString(element, "mode", this.mode_);
        }
        if (this.namelist_ != null) {
            URelaxer.setAttributePropertyByString(element, "namelist", this.namelist_);
        }
        size = this.content_.size();
        for (int i = 0;i < size;i++) {
            IVxmlFilledMixed value = (IVxmlFilledMixed)this.content_.get(i);
            value.makeElement(element);
        }
        parent.appendChild(element);
    }

    /**
     * Initializes the <code>VxmlFilled</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file.toURL());
    }

    /**
     * Initializes the <code>VxmlFilled</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(uri, UJAXP.FLAG_NAMESPACE_AWARE));
    }

    /**
     * Initializes the <code>VxmlFilled</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(url, UJAXP.FLAG_NAMESPACE_AWARE));
    }

    /**
     * Initializes the <code>VxmlFilled</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(in, UJAXP.FLAG_NAMESPACE_AWARE));
    }

    /**
     * Initializes the <code>VxmlFilled</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(is, UJAXP.FLAG_NAMESPACE_AWARE));
    }

    /**
     * Initializes the <code>VxmlFilled</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(reader, UJAXP.FLAG_NAMESPACE_AWARE));
    }

    /**
     * Creates a DOM document representation of the object.
     *
     * @exception ParserConfigurationException
     * @return Document
     */
    public Document makeDocument() throws ParserConfigurationException {
        Document doc = UJAXP.makeDocument();
        makeElement(doc);
        return (doc);
    }

    /**
     * Sets a mixed content by <code>String</code>.
     *
     * @param value
     */
    public void setContent(String value) {
        setContent(new RString(value));
    }

    /**
     * Adds a mixed content by <code>String</code>.
     *
     * @param value
     */
    public void addContent(String value) {
        addContent(new RString(value));
    }

    /**
     * Sets a mixed content by <code>String</code>.
     *
     * @param value
     */
    public void setContentByString(String value) {
        setContent(new RString(value));
    }

    /**
     * Adds a mixed content by <code>String</code>.
     *
     * @param value
     */
    public void addContentByString(String value) {
        addContent(new RString(value));
    }

    /**
     * Sets a mixed content by <code>org.w3c.dom.Node</code>.
     *
     * @param value
     */
    public void setContent(org.w3c.dom.Node value) {
        setContent(new RString(value));
    }

    /**
     * Adds a mixed content by <code>org.w3c.dom.Node</code>.
     *
     * @param value
     */
    public void addContent(org.w3c.dom.Node value) {
        addContent(new RString(value));
    }

    /**
     * Gets the RNSContext property <b>RNSContext</b>.
     *
     * @return RNSContext
     */
    public RNSContext rGetRNSContext() {
        return (rNSContext_);
    }

    /**
     * Sets the RNSContext property <b>RNSContext</b>.
     *
     * @param rNSContext
     */
    public void rSetRNSContext(RNSContext rNSContext) {
        this.rNSContext_ = rNSContext;
    }

    /**
     * Gets the String property <b>mode</b>.
     *
     * @return String
     */
    public String getMode() {
        return (mode_);
    }

    /**
     * Sets the String property <b>mode</b>.
     *
     * @param mode
     */
    public void setMode(String mode) {
        this.mode_ = mode;
    }

    /**
     * Gets the String property <b>namelist</b>.
     *
     * @return String
     */
    public String getNamelist() {
        return (namelist_);
    }

    /**
     * Sets the String property <b>namelist</b>.
     *
     * @param namelist
     */
    public void setNamelist(String namelist) {
        this.namelist_ = namelist;
    }

    /**
     * Gets the IVxmlFilledMixed property <b>content</b>.
     *
     * @return IVxmlFilledMixed[]
     */
    public IVxmlFilledMixed[] getContent() {
        IVxmlFilledMixed[] array = new IVxmlFilledMixed[content_.size()];
        return ((IVxmlFilledMixed[])content_.toArray(array));
    }

    /**
     * Sets the IVxmlFilledMixed property <b>content</b>.
     *
     * @param content
     */
    public void setContent(IVxmlFilledMixed[] content) {
        this.content_.clear();
        for (int i = 0;i < content.length;i++) {
            addContent(content[i]);
        }
        for (int i = 0;i < content.length;i++) {
            content[i].rSetParentRNode(this);
        }
    }

    /**
     * Sets the IVxmlFilledMixed property <b>content</b>.
     *
     * @param content
     */
    public void setContent(IVxmlFilledMixed content) {
        this.content_.clear();
        addContent(content);
        if (content != null) {
            content.rSetParentRNode(this);
        }
    }

    /**
     * Adds the IVxmlFilledMixed property <b>content</b>.
     *
     * @param content
     */
    public void addContent(IVxmlFilledMixed content) {
        this.content_.add(content);
        if (content != null) {
            content.rSetParentRNode(this);
        }
    }

    /**
     * Adds the IVxmlFilledMixed property <b>content</b>.
     *
     * @param content
     */
    public void addContent(IVxmlFilledMixed[] content) {
        for (int i = 0;i < content.length;i++) {
            addContent(content[i]);
        }
        for (int i = 0;i < content.length;i++) {
            content[i].rSetParentRNode(this);
        }
    }

    /**
     * Gets number of the IVxmlFilledMixed property <b>content</b>.
     *
     * @return int
     */
    public int sizeContent() {
        return (content_.size());
    }

    /**
     * Gets the IVxmlFilledMixed property <b>content</b> by index.
     *
     * @param index
     * @return IVxmlFilledMixed
     */
    public IVxmlFilledMixed getContent(int index) {
        return ((IVxmlFilledMixed)content_.get(index));
    }

    /**
     * Sets the IVxmlFilledMixed property <b>content</b> by index.
     *
     * @param index
     * @param content
     */
    public void setContent(int index, IVxmlFilledMixed content) {
        this.content_.set(index, content);
        if (content != null) {
            content.rSetParentRNode(this);
        }
    }

    /**
     * Adds the IVxmlFilledMixed property <b>content</b> by index.
     *
     * @param index
     * @param content
     */
    public void addContent(int index, IVxmlFilledMixed content) {
        this.content_.add(index, content);
        if (content != null) {
            content.rSetParentRNode(this);
        }
    }

    /**
     * Remove the IVxmlFilledMixed property <b>content</b> by index.
     *
     * @param index
     */
    public void removeContent(int index) {
        this.content_.remove(index);
    }

    /**
     * Remove the IVxmlFilledMixed property <b>content</b> by object.
     *
     * @param content
     */
    public void removeContent(IVxmlFilledMixed content) {
        this.content_.remove(content);
    }

    /**
     * Clear the IVxmlFilledMixed property <b>content</b>.
     *
     */
    public void clearContent() {
        this.content_.clear();
    }

    /**
     * Makes an XML text representation.
     *
     * @return String
     */
    public String makeTextDocument() {
        StringBuffer buffer = new StringBuffer();
        makeTextElement(buffer);
        return (new String(buffer));
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(StringBuffer buffer) {
        int size;
        String prefix = rNSContext_.getPrefixByUri("http://hil.t.u-tokyo.ac.jp/XML/2002/VXML");
        buffer.append("<");
        URelaxer.makeQName(prefix, "filled", buffer);
        rNSContext_.makeNSMappings(buffer);
        if (mode_ != null) {
            buffer.append(" ");
            buffer.append("mode");
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getMode())));
            buffer.append("\"");
        }
        if (namelist_ != null) {
            buffer.append(" ");
            buffer.append("namelist");
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getNamelist())));
            buffer.append("\"");
        }
        size = this.content_.size();
        for (int i = 0;i < size;i++) {
            IVxmlFilledMixed value = (IVxmlFilledMixed)this.content_.get(i);
            value.makeTextAttribute(buffer);
        }
        buffer.append(">");
        size = this.content_.size();
        for (int i = 0;i < size;i++) {
            IVxmlFilledMixed value = (IVxmlFilledMixed)this.content_.get(i);
            value.makeTextElement(buffer);
        }
        buffer.append("</");
        URelaxer.makeQName(prefix, "filled", buffer);
        buffer.append(">");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextElement(Writer buffer) throws IOException {
        int size;
        String prefix = rNSContext_.getPrefixByUri("http://hil.t.u-tokyo.ac.jp/XML/2002/VXML");
        buffer.write("<");
        URelaxer.makeQName(prefix, "filled", buffer);
        rNSContext_.makeNSMappings(buffer);
        if (mode_ != null) {
            buffer.write(" ");
            buffer.write("mode");
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getMode())));
            buffer.write("\"");
        }
        if (namelist_ != null) {
            buffer.write(" ");
            buffer.write("namelist");
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getNamelist())));
            buffer.write("\"");
        }
        size = this.content_.size();
        for (int i = 0;i < size;i++) {
            IVxmlFilledMixed value = (IVxmlFilledMixed)this.content_.get(i);
            value.makeTextAttribute(buffer);
        }
        buffer.write(">");
        size = this.content_.size();
        for (int i = 0;i < size;i++) {
            IVxmlFilledMixed value = (IVxmlFilledMixed)this.content_.get(i);
            value.makeTextElement(buffer);
        }
        buffer.write("</");
        URelaxer.makeQName(prefix, "filled", buffer);
        buffer.write(">");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(PrintWriter buffer) {
        int size;
        String prefix = rNSContext_.getPrefixByUri("http://hil.t.u-tokyo.ac.jp/XML/2002/VXML");
        buffer.print("<");
        URelaxer.makeQName(prefix, "filled", buffer);
        rNSContext_.makeNSMappings(buffer);
        if (mode_ != null) {
            buffer.print(" ");
            buffer.print("mode");
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getMode())));
            buffer.print("\"");
        }
        if (namelist_ != null) {
            buffer.print(" ");
            buffer.print("namelist");
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getNamelist())));
            buffer.print("\"");
        }
        size = this.content_.size();
        for (int i = 0;i < size;i++) {
            IVxmlFilledMixed value = (IVxmlFilledMixed)this.content_.get(i);
            value.makeTextAttribute(buffer);
        }
        buffer.print(">");
        size = this.content_.size();
        for (int i = 0;i < size;i++) {
            IVxmlFilledMixed value = (IVxmlFilledMixed)this.content_.get(i);
            value.makeTextElement(buffer);
        }
        buffer.print("</");
        URelaxer.makeQName(prefix, "filled", buffer);
        buffer.print(">");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(StringBuffer buffer) {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextAttribute(Writer buffer) throws IOException {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(PrintWriter buffer) {
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getModeAsString() {
        return (URelaxer.getString(getMode()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getNamelistAsString() {
        return (URelaxer.getString(getNamelist()));
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setModeByString(String string) {
        setMode(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setNamelistByString(String string) {
        setNamelist(string);
    }

    /**
     * Returns a String representation of this object.
     * While this method informs as XML format representaion, 
     *  it's purpose is just information, not making 
     * a rigid XML documentation.
     *
     * @return String
     */
    public String toString() {
        try {
            return (makeTextDocument());
        } catch (Exception e) {
            return (super.toString());
        }
    }

    /**
     * Accepts the Visitor for enter behavior.
     *
     * @param visitor
     * @return boolean
     */
    public boolean enter(IRVisitor visitor) {
        return (visitor.enter(this));
    }

    /**
     * Accepts the Visitor for leave behavior.
     *
     * @param visitor
     */
    public void leave(IRVisitor visitor) {
        visitor.leave(this);
    }

    /**
     * Gets the IRNode property <b>parentRNode</b>.
     *
     * @return IRNode
     */
    public IRNode rGetParentRNode() {
        return (parentRNode_);
    }

    /**
     * Sets the IRNode property <b>parentRNode</b>.
     *
     * @param parentRNode
     */
    public void rSetParentRNode(IRNode parentRNode) {
        this.parentRNode_ = parentRNode;
    }

    /**
     * Gets child RNodes.
     *
     * @return IRNode[]
     */
    public IRNode[] rGetRNodes() {
        java.util.List classNodes = new java.util.ArrayList();
        classNodes.addAll(content_);
        IRNode[] nodes = new IRNode[classNodes.size()];
        return ((IRNode[])classNodes.toArray(nodes));
    }

    /**
     * Tests if a Element <code>element</code> is valid
     * for the <code>VxmlFilled</code>.
     *
     * @param element
     * @return boolean
     */
    public static boolean isMatch(Element element) {
        if (!URelaxer2.isTargetElement(element, "http://hil.t.u-tokyo.ac.jp/XML/2002/VXML", "filled")) {
            return (false);
        }
        RStack target = new RStack(element);
        boolean $match$ = false;
        Element child;
        if (RString.isMatch(target)) {
            $match$ = true;
        }
        while (true) {
            if (VxmlSubmit.isMatchHungry(target)) {
                $match$ = true;
            } else if (VxmlGoto.isMatchHungry(target)) {
                $match$ = true;
            } else if (VxmlAudio.isMatchHungry(target)) {
                $match$ = true;
            } else if (VxmlPrompt.isMatchHungry(target)) {
                $match$ = true;
            } else if (VxmlScript.isMatchHungry(target)) {
                $match$ = true;
            } else if (VxmlThrow.isMatchHungry(target)) {
                $match$ = true;
            } else if (VxmlLog.isMatchHungry(target)) {
                $match$ = true;
            } else if (VxmlNative.isMatchHungry(target)) {
                $match$ = true;
            } else if (VxmlAssign.isMatchHungry(target)) {
                $match$ = true;
            } else if (VxmlExit.isMatchHungry(target)) {
                $match$ = true;
            } else if (VxmlReturn.isMatchHungry(target)) {
                $match$ = true;
            } else if (VxmlVar.isMatchHungry(target)) {
                $match$ = true;
            } else if (VxmlValue.isMatchHungry(target)) {
                $match$ = true;
            } else if (VxmlClear.isMatchHungry(target)) {
                $match$ = true;
            } else if (VxmlIf.isMatchHungry(target)) {
                $match$ = true;
            } else if (VxmlEnumerate.isMatchHungry(target)) {
                $match$ = true;
            } else if (VxmlDisconnect.isMatchHungry(target)) {
                $match$ = true;
            } else if (VxmlReprompt.isMatchHungry(target)) {
                $match$ = true;
            } else {
                break;
            }
        }
        if (!target.isEmptyElement()) {
            return (false);
        }
        return (true);
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>VxmlFilled</code>.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatch(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        return (isMatch(element));
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>VxmlFilled</code>.
     * This method consumes the stack contents during matching operation.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatchHungry(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        if (isMatch(element)) {
            stack.popElement();
            return (true);
        } else {
            return (false);
        }
    }
}
